;+
;Procedure: THM_CAL_EFI
;
;Purpose:  Converts raw EFI (V, EDC, and EAC waveform) data into physical quantities.
;
;keywords:
;   /VERBOSE or VERBOSE=n ; set to enable diagnostic message output.
;		higher values of n produce more and lower-level diagnostic
;		messages.
;   datatype  - default setting is to calibrate all raw quantites and also
;               produce all _0 and _dot0 quantities.
;   probe     - defaults to all probes
;   valid_names - print valid datatypes
;   coord     - coordinate system of output.
;keyword parameters for _dot0 computation:
;   max_angle - maximum angle of B field to spin axis to calculate _dot0. 
;               defaults to 80 degrees.
;   min_bz    - minimum value of Bz.  Typical value is 2.0 nT.  No default.
;               Not compatible with max_angle keyword.
;   max_bxy_bz- maximum value of abs(bx/bz) or abs(by/bz).  Typical value is 5.
;               Default is not to use this method.
;   bz_offset - offset in nT that will be added to Z axis measurement of B.
;   fgm_datatype - 'fgl', 'fgh', 'fgs' or 'fge'
;
;Example:
;   thm_cal_efi, probe='c'
;
;Notes:
;	-- fixed, nominal calibration pars used (gains and frequency responses), rather than proper time-dependent parameters.
;
; $LastChangedBy: jimm $
; $LastChangedDate: 2007-10-22 13:20:13 -0700 (Mon, 22 Oct 2007) $
; $LastChangedRevision: 1789 $
; $URL $
;-
pro thm_cal_efi, probe=probe, datatype=datatype, files=files,trange=trange,$
                 valid_names=valid_names, verbose=verbose, coord=coord, $
                 in_suffix=in_suf,out_suffix=out_suf, max_angle=max_angle, $
                 min_bz=min_bz, max_bxy_bz=max_bxy_bz, bz_offset=bz_offset, $
                 fgm_datatype=fgm_datatype

  thm_init
  vb = size(verbose, /type) ne 0 ? verbose : !themis.verbose

  if not keyword_set(coord) then coord='dsl'
  primary_datatypes = [ 'vaf', 'vap', 'vaw', 'vbf', 'vbp', 'vbw', 'eff', $
                        'efp', 'efw'] ; note that efi_dq is deleted.
  u0s=['eff_0','efp_0','efw_0']
  udot0s=['eff_dot0','efp_dot0','efw_dot0']
  efi_valid_names = [primary_datatypes, u0s, udot0s]

  if not keyword_set(in_suf) then in_suf=''
  if not keyword_set(out_suf) then out_suf=''
  if arg_present( valid_names) then begin
     valid_names = efi_valid_names
     message, /info, string( strjoin( efi_valid_names, ','), $
                             format='( "Valid names:",X,A,".")')
     return
  endif

  vprobes = ['a','b','c','d','e']
  if n_elements(probe) eq 1 then if probe eq 'f' then vprobes = ['f']
  if not keyword_set(probe) then probes = vprobes $
  else probes = thm_check_valid_name(strlowcase(probe), vprobes, /include_all)
  if not keyword_set(probes) then return
  if keyword_set(vb) then printdat, probes, /value, varname='Probes'

  if not keyword_set(datatype) then dts = efi_valid_names $
  else dts = thm_check_valid_name(strlowcase(datatype), efi_valid_names, $
                                  /include_all)
  if not keyword_set(dts) then return
  if keyword_set(vb) then printdat, dts, /value, varname='Datatypes'


if keyword_set(max_angle) and keyword_set(min_bz) then begin
   message, /cont, 'min_bz and max_angle keywords are mutually exclusive'
   return
endif
if not arg_present(bz_offset) then bz_offset = 0.0                 ; nT
if not arg_present(max_angle) then max_angle = 80
;         min_bz = 2.0 ; nT
;         max_bxy_bz = 5.0

for s=0L,n_elements(probes)-1L do begin
   sc = probes[s]
   for n=0L, n_elements(dts)-1L do begin
      name = dts[ n]
      if where(name eq u0s or name eq udot0s) ne -1 then begin
         nameraw=strmid(name,0,3)
      endif else nameraw=name
      tplot_var_raw = thm_tplot_var( sc, nameraw)+in_suf
      tplot_var = thm_tplot_var( sc, name)+out_suf
      get_data, tplot_var_raw, data=d, limit=l, dlim=dl
      
      if keyword_set( vb) then $
         message, /cont, string( tplot_var, $
                                 format='("working on TPLOT variable",X,A)')

      ;; check to see if the data has already been calibrated
      if( thm_data_calibrated( dl)) then begin
         if nameraw eq name then begin
            if keyword_set( vb) then begin
               message, /cont, string( tplot_var, format= $
                                       '(A,X,"has already been calibrated.")')
            endif
         endif else begin
            thm_get_efi_cal_pars, tbeg, tend, nameraw, cal_pars=cp
            thm_cotrans, tplot_var_raw, tplot_var, out_coord = 'dsl'
         endelse
      endif else begin ;; otherwise, proceed with the RAW->PHYS transformation.
         hed_tplot_var=thm_tplot_var(sc,nameraw)
         tplot_var_hed = hed_tplot_var + '_hed'
         get_data, tplot_var_hed, data=d_hed, limit=l_hed, dlim=dl_hed

         ;; check that returned data and hed structures are structures
         ;; (get_data returns 0 if no TPLOT variable exists).
         if (size( d, /type) eq 8) and (size( d_hed, /type) eq 8) then begin
           
            hed_data = thm_unpack_hed( name, d_hed.y)
            switch strlowcase( name) of
               'vaf':
               'vbf':
               'vap':
               'vbp':
               'vaw':
               'vbw':	begin
                  tbeg = min( d.x, max=tend)
                  thm_get_efi_cal_pars, tbeg, tend, name, cal_pars=cp
                                ;stop
                  res = fltarr( size( d.y, /dimensions))
                  for icomp=0L,5L do begin
                     res[ *, icomp] = cp.gain[ icomp]*(float(d.y[ *, icomp]) - $
                                                       cp.offset[ icomp])
                  endfor
                                ;stop

                  ;; update the DLIMIT elements to reflect RAW->PHYS 
                  ;; transformation, coordinate system, etc.

                  units = cp.units
                 
                  str_element, dl, 'data_att', data_att, success=has_data_att
                  if has_data_att then begin
                     str_element, data_att, 'data_type', 'calibrated', /add
                  endif else data_att = { data_type: 'calibrated' }
                  str_element, data_att, 'coord_sys',  'efi_sensor', /add
                  str_element, data_att, 'units', units, /add
                  str_element, data_att, 'cal_par_time', cp.cal_par_time, /add
                  str_element, dl, 'data_att', data_att, /add
                  
                  str_element, dl, 'ytitle', string(tplot_var, units, format= $
                                                    '(A,"!C!C[",A,"]")'), /add
                  str_element, dl, 'units', units, /add
                  str_element, dl, 'labels', $
                               [ 'V1', 'V2', 'V3', 'V4', 'V5', 'V6'], /add
                  str_element, dl, 'labflag', 1, /add
                  str_element, dl, 'colors', [ 1, 2, 3, 4, 5, 6]
                  
                  ;; store the transformed spectra back into the original 
                  ;; TPLOT variable.
                  store_data, tplot_var, $
                              data = { x:d.x, y:res, v:d.v }, $
                              lim=l, dlim=dl
                 
                  break
               end  ; end of { VAF, VBF, VAP, VBP, VAW, VBW} calibration clause.
               'eff':
               'efp':
               'efw':
               'efp_0':
               'eff_0':
               'efw_0':
               'efp_dot0':
               'efw_dot0':
               'eff_dot0': begin ; { EFF, EFP, EFW} calibration clause.
                 
                  tbeg = min( d.x, max=tend)
                  thm_get_efi_cal_pars, tbeg, tend, nameraw, cal_pars=cp
                                ;stop
                  e12=49.6
                  e34=40.4
                  e56=5.63
                  res = fltarr( size( d.y, /dimensions))
                  for icomp=0L,2L do begin
                     if icomp eq 0 then exx=e12
                     if icomp eq 1 then exx=e34
                     if icomp eq 2 then exx=e56
                     res[ *, icomp] = -1000*cp.gain[ icomp] * $
                                      (d.y[ *, icomp] - cp.offset[ icomp])/exx
                     
                  endfor
                  ;; update the DLIMIT elements to reflect RAW->PHYS
                  ;; transformation, coordinate system, etc.
                  cp.units='mV/m'
                  units = cp.units
                  str_element, dl, 'data_att', data_att, success=has_data_att
                  if has_data_att then begin
                     str_element, data_att, 'data_type', 'calibrated', /add
                  endif else data_att = { data_type: 'calibrated' }
                  str_element, data_att, 'coord_sys',  'spg', /add
                  str_element, data_att, 'units', units, /add
                  str_element, data_att, 'cal_par_time', cp.cal_par_time, /add
                  
                  str_element, dl, 'data_att', data_att, /add
                 
                  str_element, dl, 'ytitle', string( tplot_var, units, format= $
                                                     '(A,"!C!C[",A,"]")'), /add

                  str_element, dl, 'labels', [ 'E12', 'E34', 'E56'], /add
                  str_element, dl, 'labflag', 1, /add
                  str_element, dl, 'colors', [ 2, 4,6]

                  ;; store the transformed spectra into the output
                  ;; TPLOT variable.
                  store_data, tplot_var, $
                              data = { x:d.x, y:res, v:d.v }, $
                              lim=l, dlim=dl
                  
                  ;; transfrom _dot0 or _0  quantities to dsl for further 
                  ;; processing
                  if where(name eq [udot0s, u0s]) ge 0 then begin
                     thm_cotrans, tplot_var, out_coord='dsl'
                  endif else if keyword_set(coord) && coord ne 'spg' then begin
                     thm_cotrans,tplot_var,out_coord=coord
                  endif

                  break
               end              ; of { EFF, EFP, EFW} calibration clause.
               else:	begin   ; improperly defined name of EFI quantity.
               end
            endswitch
         endif else begin	; necessary TPLOT variables not present.
            if keyword_set(vb) then $
               message, /continue, $
                        string( tplot_var_raw, tplot_var_hed, format= $
                                '("necessary TPLOT variables (",A,X,A,") ' + $
                                'not present for RAW->PHYS transformation.")')
            
         endelse
      endelse                   ; done with RAW->PHYS transformation
                                ;stop


      ;; Finish processing of _0 quantities
      if where(name eq u0s) ne -1 then begin
         get_data, tplot_var, data=d, limit=l, dlim=dl
         if size(d, /type) ne 8 then continue
         res = d.y
         res[*,2]=0
         store_data, tplot_var, $
                     data = { x:d.x, y:res, v:d.v }, $
                     lim=l, dlim=dl

         if keyword_set(coord) && coord ne 'dsl' then begin
            if coord ne 'spg' and coord ne 'ssl' then begin
               message, /continue, $
                        'Warning: coord keyword ignored: ' + $
                        'only ssl, dsl, or spg are physically meaningful ' + $
                        'coordinate systems for _0 quantites.'
            endif else begin
               thm_cotrans,tplot_var,out_coord=coord
            endelse
         endif
      endif
      
      ;; Finish processing of _dot0 quantities
      if where(name eq udot0s) ne -1 then begin
         get_data, tplot_var, data=d, limit=l, dlim=dl
         if size(d, /type) ne 8 then continue
         res = d.y

         suff = '_thm_cal_efi_priv'
         thm_load_fgm,probe=sc,level='l2',coord='dsl', suffix = suff
         if keyword_set(fgm_datatype) then ftype = fgm_datatype $
         else begin
            if nameraw eq 'efp' || nameraw eq 'efw' $
            then ftype='fgh' $
            else ftype='fgl'
         endelse 

         if vb ge 2 then print, 'Using '+ftype

         tdegap, 'th'+sc+'_'+ftype+'_dsl'+suff, dt = cp.cal_par_time, $
                 margin = cp.cal_par_time * 0.2, /overwrite

         get_data,'th'+sc+'_'+ftype+'_dsl'+suff, $
                  data=thx_fgx_dsl, limit=fl, dlim=fdl
         
         del_data, '*'+suff

         Bx = interpol(thx_fgx_dsl.y[*,0],thx_fgx_dsl.x, d.x)
         By = interpol(thx_fgx_dsl.y[*,1],thx_fgx_dsl.x, d.x)
         Bz = interpol(thx_fgx_dsl.y[*,2],thx_fgx_dsl.x, d.x) + bz_offset
         
         res[*,2] *= !values.f_nan

         if keyword_set(max_angle) then begin
            angle=acos(Bz/sqrt(Bx^2+By^2+Bz^2))*180/!dpi
            good = where(abs(angle) le max_angle, ngood)
            if ngood eq 0 then continue
         endif else if keyword_set(min_bz) then begin
            good = where(abs(Bz) ge min_bz, ngood)
            if ngood eq 0 then continue
         endif else good = lindgen(n_elements(d.x))

         bx_bz = Bx[good]/Bz[good]
         by_bz = By[good]/Bz[good]

         Ez = -(bx_bz)*res[good,0] -(by_bz)*res[good,1]

         if keyword_set(max_bxy_bz) then begin
            keep = where(abs(bx_bz) lt max_bxy_bz and abs(by_bz) lt max_bxy_bz,$
                        nkeep)
            if nkeep eq 0 then continue
         endif else keep = lindgen(ngood)

         res[good[keep],2]=Ez[keep]

         store_data, tplot_var, $
                     data = { x:d.x, y:res, v:d.v }, $
                     lim=l, dlim=dl

         if keyword_set(coord) && coord ne 'dsl' then begin
            thm_cotrans,tplot_var,out_coord=coord
         endif

      endif

   endfor                       ; loop over names.
endfor                          ; loop over spacecraft.

end
